% Program to do construct the lesat favourable test for the Behrens Fisher
% problem in Elliott, Mueller and Watson (2013). This version is based on 
% the raw data for x1 and x2 and
% conducts the test H0: beta=0, delta  vs H1: beta non zero and
% delta. 
% 
% 
%
% INPUTS
% x1          :   data for the first variable (shorter data series)
% x2          :   data for the second variable (longer data series)
%
% OUTPUTS     
% rejf        :   flag = 1 for rejection, zero otherwise
% note: use the shorter data series for x2, if equal length either ordering
% works


function [rejf,sw] = BehrensFisher(x1,x2);

% construct the basic statistics and parameters

n1=length(x1);
n2=length(x2);
t=(mean(x1)-mean(x2))/std(x2);
r=std(x1)/std(x2);
yb=(mean(x1)-mean(x2))/sqrt(((std(x1)^2)/n1)+((std(x2)^2)/n2));
sw=0;
% for the switching
if log(r)>6; sw=1; 
    rejf=(abs(yb)>tinv(0.975,n1-1));
elseif log(r)<-6;sw=1;
    rejf=(abs(yb)>tinv(0.975,n2-1));
else
    
    % for the statistic when we do not switch

if n1==2; betalt=4; else; betalt=3; end;
% for the likelihood under the alternative
lrni=zeros(2,1);
q1=integral(@(x)bffn(t,r,-1*betalt,x,n1,n2),-9,9);
lrni(1,1)=log(0.5)+log(q1)-log(18);
q2=integral(@(x)bffn(t,r,betalt,x,n1,n2),-9,9);
lrni(2,1)=log(0.5)+log(q2)-log(18);


m=max(lrni);
lrn=m+log(sum(exp(lrni-m)));
% this just sums the two parts

% for the likelihood under the null
% first calculate the weights and critical value given rho


% load weights
load bfmat;
% program is a 23x51 matrix. First column (apart from first two rows) gives
% values for sample sizes. Columns are for ranges to integrate over delta (apart
% from the first column). The first two rows given the lower and upper
% bounds for the integration over delta, the remainder of the tables are
% the weights
% first, select correct row based on sample sizes
wmat=bfmat(1:2,2:51)';       % gives the ranges for delta
if n1==2; 
    p=bfmat(n2+1,2:51)';      % gives weights when n1=2
else
    p=bfmat(n2+11,2:51)';     % gives weights when n1=3
end;


lrdi=zeros(length(p),1);
for ib=1:length(p);
    
del0=wmat(ib,1);
del1=wmat(ib,2);
qi=integral(@(x)bffn(t,r,0,x,n1,n2),del0,del1);
lrdi(ib,1)=log(p(ib,1))+log(qi)-log(del1-del0);

end;

    m=max(lrdi);
    lrd=m+log(sum(exp(lrdi-m)));
tst=lrn-lrd;
rejf=tst>0;

end;


