% Main function to compute the LFAV break data test statistic. This version
% imposes variance=1. Likelihood under the null (beta=b0) and alternative
% are computed seperately. This version requires the null break date to be
% in the first half of the sample, if not reverse the sample
% 
% INPUTS:
% y        : nx1 vector of data to test for the break date
% b0       : proportion between 0.15 and 0.5 for the location of the null
%            break date
%
% OUTPUTS
% rejf     : flag =1 for rejection, zero otherwise
%
% format [rejf] = breakdate(y,b0);

function [rejf] = breakdate(y,b0)


n=length(y); 
s2d=100;                    %  variance for delta under Ha
ym=(y-mean(y));             % remove mean and normalize
%Gb=recserar(ym,0,1)/(std(ym)*sqrt(n));        %  normalized Brownian Bridge from data under H0
Gb=recserar(ym,0,1)/(sqrt(n));
% for the likelihood under the alternative

ll=floor(n*0.15);
ul=ceil(n*0.85);
bet=((ll:1:ul)')/n; 
betd=bet.*(1-bet);
t1=sqrt(1+s2d*betd);
t2=((Gb(ll:ul,1).^2)./(2*((1/s2d)+betd)));
t3=((1/s2d)+betd)-(Gb(ll:ul,1)./sqrt((1/s2d)+betd));


% use the sums of logs trick to enhance accuracy
lrni=t2+log(normcdf(20*t3)-normcdf(-20*t3))-log(t1)-log(normcdf(2)-normcdf(-2));
m=max(lrni);
lrn=m+log(mean(exp(lrni-m)));
% note : taking the mean ensures the equal weights over beta, also makes
% divisor =0.7


% for the likelihood under the null

% first calculate the weights for the stated null beta0
load breakm;
% this is 38 x 21. Has delta lower bar and upper bar in the first two rows,
% starting at the second column. Apart from first two rows, first column
% has values for beta0 from 0.15 to 0.5 (for a total of 36 values). The
% remaining entries are weights for each of the delta ranges
wmat=breakm(1:2,2:21)';
p=crlk(b0,breakm(3:38,:))';    % for beta0 from 0.15 to 0.5

lrdi=zeros(2*length(p),1);

%p=p/sum(p);
p=[0.5*p;0.5*p];
dm1=[-wmat(:,2) -wmat(:,1)];
dmat=[wmat(:,1:2); dm1];
  


Gb0=Gb(floor(n*b0),1);
betd0=b0*(1-b0);
t1=(Gb0^2)/(2*betd0);

for ib=1:length(dmat);
    
del0=dmat(ib,1);
del1=dmat(ib,2);

d0=del0*sqrt(betd0);
d1=del1*sqrt(betd0);


if d1-Gb0/sqrt(betd0)<0;
    normp=normcdf(d1-Gb0/sqrt(betd0))-normcdf(d0-Gb0/sqrt(betd0)); 
elseif d0-Gb0/sqrt(betd0)>0;
    normp=normcdf(-d0+Gb0/sqrt(betd0))-normcdf(-d1+Gb0/sqrt(betd0));
else
    normp=1-normcdf(-d1+Gb0/sqrt(betd0))-normcdf(d0-Gb0/sqrt(betd0));
end;

lrdi(ib,1)=(log(p(ib,1))+t1+0.5*log(2*pi/betd0)+log(normp)-log(del1-del0));

end;

m=max(lrdi);
lrd=m+log(sum(exp(lrdi-m)));
%[lrn lrd lrn-lrd cv]

tst=lrn-lrd;
rejf=tst>0;
