% Program to do construct the lesat favourable test for the Set-Identified
% parameter problem in Elliott, Mueller and Watson (2013). This version
% takes rho as known as well as the pair of observations (y1,y2) and
% conducts the test H0: beta=0, delta non negative vs H1: beta non zero and
% delta nonnegative. 
% 
% 
%
% INPUTS
% y1          :   lower end estimate Yl
% y2          :   upper end estimate Yu
% rho         :   value for rho
%
% OUTPUTS     
% rejf        :   flag = 1 for rejection, zero otherwise



function [rejf] = imbman(y1,y2,rho);

y=[y1;y2];    % these are Yl and Yu

if (y2-y1)>6;
    rejf=(y1>1.645)+(y2<-1.645);
else

sig=eye(2);
sig(1,2)=rho;
sig(2,1)=sig(1,2);

e2=[0;1];
i2=(e2'*(sig\e2));



% for the likelihood under the alternative
lrni=zeros(2,1);
for ib=1:2;
if ib==1;
   ya=y-2;
   del0=0;
   del1=8;
else
   ya=[y(2,1)+2;y(1,1)+2];   % reverse ordering to make the calculations simpler
   del0=-8;
   del1=0;
end;
dbar=e2'*(sig\ya)/i2;
t1=(-0.5*ya'*(sig\ya)+0.5*(dbar'*dbar)*i2);
if sqrt(i2)*(del1-dbar)<0;
    norma=normcdf(sqrt(i2)*(del0-dbar));
    normb=normcdf(sqrt(i2)*(del1-dbar));
else
    norma=normcdf(-sqrt(i2)*(del1-dbar));
    normb=normcdf(-sqrt(i2)*(del0-dbar));
end;

lrni(ib,1)=log(0.5)+t1+log(sqrt(2*pi/i2))+log(normb-norma)-log(8);

end;
m=max(lrni);
lrn=m+log(sum(exp(lrni-m)));
% this just sums the two parts

% for the likelihood under the null
% first calculate the weights and critical value given rho


% load weights
load partialm;
% program is a 33x28 matrix. First column (apart from first two rows) gives
% values for rho. Columns are for ranges to integrate over delta (apart
% from the first column). The first two rows given the lower and upper
% bounds for the integration over delta, the remainder of the tables are
% the weights
% first, select correct row based on rho
wmat=partialm(1:2,2:28)';       % gives the ranges for delta
p=crlk(rho,partialm(3:33,:))';    % gives the weights for each range

lrdi=zeros(2*length(p),1);
% pad out wmat for the positive and negative sides (values for tau)
p=[0.5*p;0.5*p];
dm1=[-wmat(:,2) -wmat(:,1)];
dmat=[wmat(:,1:2); dm1];

for ib=1:length(dmat);
    
del0=dmat(ib,1);
del1=dmat(ib,2);

if ib>length(wmat);
    ya=[y2;y1];     % reverse thetal and thetau for the second set
else
    ya=[y1;y2];
end;
dbar=e2'*(sig\ya)/i2;
t1=(-0.5*ya'*(sig\ya)+0.5*(dbar*dbar)*i2);

if sqrt(i2)*(del1-dbar)<0;
    normp=normcdf(sqrt(i2)*(del1-dbar))-normcdf(sqrt(i2)*(del0-dbar)); 
else
    % sqrt(i2)*(del0-dbar)>0;
    normp=normcdf(-sqrt(i2)*(del0-dbar))-normcdf(-sqrt(i2)*(del1-dbar));

end;


lrdi(ib,1)=log(p(ib,1))+t1+log(sqrt(2*pi/i2))+log(normp)-log(del1-del0);

end;

m=max(lrdi);
lrd=m+log(sum(exp(lrdi-m)));
tst=lrn-lrd;
rejf=tst>0;

end;


