% Version of predictive regression test. This version has point weights on various delta 
% Test is one sided, Ha beta>0
% 
% Inputs
% Y1,Y2,Y3,Y4 as in the paper after equation (25)
% rho is the correlation 
% Outputs
% rejf is a flag=1 for rejection, zero for fail to reject
% tst is the value of the test statistic
%
% Note this can be used for asymptotic and small sample using relevant
% inputs

function [rejf] = predreg1side(y1,y2,y3,y4,rho)

  r1=1/sqrt(1-rho*rho);
  dhat=-(y2+rho*r1*y1)/y4;
 
 if dhat>=130;
    bhat=(y1/y3)-r1*dhat;
    wld=(bhat)/sqrt((1/y3)+r1*r1/y4);
    if rho<0;
        rejf=wld>1.70;
    else
        rejf=wld>1.645;
    end;

    
 else
     
load predlambda1;
[nr,nl]=size(predlambda1);

% predlambda1 has first row giving values for delta for weights. First
% column gives rho and then the weights for each rho

% first calculate the lambda weights for a given rho
p=crlk(rho,predlambda1(2:nr,:));
%p=p./sum(p);

dltv=predlambda1(1,2:nl)';
nd=length(dltv);
lrdi=zeros(nd,1);   
lrni=lrdi;


for ib=1:nd;
    dlt=dltv(ib,1);
    b2=(rho*dlt*r1)^2;
    bet=1.645*sqrt(2*dlt+6)*r1;
    b1=(bet+rho*dlt*r1)^2;
    if dlt<160;
    lrni(ib,1)=log(1/(nd-6))+bet*y1-dlt*y2-0.5*b1*y3-0.5*dlt*dlt*y4;
    end;
    lrdi(ib,1)=log(p(1,ib))-dlt*y2-0.5*b2*y3-0.5*dlt*dlt*y4;
    
end;

lrni=lrni(1:nd-6,1);

m=max(lrdi);
lrd=m+log(sum(exp(lrdi-m)));
m=max(lrni);
lrn=m+log(sum(exp(lrni-m)));


tst=lrn-lrd;
rejf=tst>0;
end;
