% Program to do construct the lesat favourable test for the regressor
% selection problem in Elliott, Mueller and Watson (2013). This version
% takes rho as known as well as the pair of observations (y1,y2) and
% conducts the test H0: beta=0, delta free vs H1: beta non zero and
% delta zero. 
% 
% 
%
% INPUTS
% y1          :   estimate for beta 
% y2          :   estimate for delta 
% rho         :   value for rho
%
% OUTPUTS     
% rejf        :   flag = 1 for rejection, zero otherwise

function [rejf] = regsel1(y1,y2,rho)

if abs(y2)>6;     % switch to simple test
    rejf=(abs(y1)>1.96);
else             % do the least favourable test
    
y=[y1;y2];

sig=eye(2);
sig(1,2)=rho;
sig(2,1)=sig(1,2);

% for the likelihood under the alternative
% we compute the likelihood for the full vector.
betalt=[-2;2];
ya=y;
lrni=zeros(2,1);
for ib=1:2;
  ya(1,1)=y(1,1)-betalt(ib,1);
  lrni(ib,1)=log(0.5)-log(2*pi)-0.5*log(det(sig))-0.5*ya'*(sig\ya);
end;
m=max(lrni);
lrn=m+log(sum(exp(lrni-m)));


% for the likelihood under the null
% load weights
load regselm;
% program is a 33x47 matrix. First column (apart from first two rows) gives
% values for rho. Columns are for ranges to integrate over delta (apart
% from the first column). The first two rows given the lower and upper
% bounds for the integration over delta, the remainder of the tables are
% the weights
% first, select correct row based on rho
wmat=regselm(1,2:47)';           % gives the values for delta
p=crlk(rho,regselm(3:33,:))';    % gives the weights for each range

lrdi=zeros(2*length(p),1);
% pad out wmat for the positive and negative sides (values for delta)
p=[0.5*p;0.5*p];
dm1=-1*wmat(:,1);
dmat=[wmat(:,1); dm1];

ya=y;      % the original y vector
for ib=1:length(dmat);
  ya(2,1)=y(2,1)-dmat(ib,1);
  lrdi(ib,1)=log(p(ib,1))-log(2*pi)-0.5*log(det(sig))-0.5*ya'*(sig\ya);
end;

m=max(lrdi);
lrd=m+log(sum(exp(lrdi-m)));
tst=lrn-lrd;

rejf=tst>0;
end;
