% Program to do construct the least favourable test for the regressor
% with known sign in Elliott, Mueller and Watson (20XX). This version
% takes rho as known as well as the pair of observations (y1,y2) and
% conducts the test H0: beta=0, delta>=0 vs Ha:  beta non zero and
% delta >=0. 
% 
% This program does the LFAV test, not including the switching
%
% INPUTS
% y1          :   estimate for beta 
% y2          :   estimate for delta 
% rho         :   value for rho
%
% OUTPUTS     
% rejf        :   flag = 1 for rejection, zero otherwise



function [rejf] = runnex(y1,y2,rho)

%if y2>6;                     % switching rul
%    rejf=(abs(y1)>1.96);     % 'simple' test
%else
    % least favourable test
y=[y1;y2];

sig=eye(2);
sig(1,2)=rho;
sig(2,1)=sig(1,2);

e2=[0;1];
i2=(e2'*(sig\e2));



% for the likelihood under the alternative

del0=0;
del1=9;
betalt=[-2;2];
ya=y;

lrni=zeros(2,1);
for ib=1:2;
ya(1,1)=y(1,1)-betalt(ib,1);
dbar=e2'*(sig\ya)/i2;
t1=(-0.5*ya'*(sig\ya)+0.5*(dbar'*dbar)*i2);
if sqrt(i2)*(del1-dbar)<0;
    norma=normcdf(sqrt(i2)*(del0-dbar));
    normb=normcdf(sqrt(i2)*(del1-dbar));
else
    norma=normcdf(-sqrt(i2)*(del1-dbar));
    normb=normcdf(-sqrt(i2)*(del0-dbar));
end;

lrni(ib,1)=log(0.5)+t1+log(normb-norma)-log(9);
end;

m=max(lrni);
lrn=m+log(sum(exp(lrni-m)));




% for the likelihood under the null
% load weights
load runnexm;
% program is a 33x27 matrix. First column (apart from first two rows) gives
% values for rho. Columns are for ranges to integrate over delta (apart
% from the first column). The first two rows given the lower and upper
% bounds for the integration over delta, the remainder of the tables are
% the weights
% first, select correct row based on rho
wmat=runnexm(1:2,2:27)';       % gives the ranges for delta
p=crlk(rho,runnexm(3:33,:))';    % gives the weights for each range
% first calculate the weights and critical value given rho


lrdi=zeros(length(wmat),1);
for ib=1:length(wmat);
    
    
del0=wmat(ib,1);
del1=wmat(ib,2);

dbar=e2'*(sig\y)/i2;
t1=(-0.5*y'*(sig\y)+0.5*(dbar'*dbar)*i2);
if sqrt(i2)*(del1-dbar)<0;
    norma=normcdf(sqrt(i2)*(del0-dbar));
    normb=normcdf(sqrt(i2)*(del1-dbar));
else
    norma=normcdf(-sqrt(i2)*(del1-dbar));
    normb=normcdf(-sqrt(i2)*(del0-dbar));
end;

lrdi(ib,1)=log(p(ib,1))+t1+log((normb-norma))-log(del1-del0);

end;
m=max(lrdi);
lrd=m+log(sum(exp(lrdi-m)));


tst=lrn-lrd;
rejf=tst>0;

%end;
